from airflow.models import DAG
from airflow.contrib.operators.bigquery_operator import BigQueryOperator
from datetime import datetime

DEFAULT_DATE = datetime(2015, 1, 1)
default_args = {'owner': 'de-team', 'start_date': DEFAULT_DATE}
DAG_NAME = 'test_read_dag_scripts'
dag = DAG(DAG_NAME, default_args=default_args)

test_read_dag_scripts = BigQueryOperator(
    task_id='test_read_dag_scripts',
    bql='select * from test_table',
    dag=dag,
    default_args=default_args,
    schema_update_options=None
)



from airflow.models import DAG
from airflow.contrib.operators.bigquery_operator import BigQueryOperator
from datetime import datetime

DEFAULT_DATE = datetime(2015, 1, 1)
default_args = {'owner': 'de-team', 'start_date': DEFAULT_DATE}
DAG_NAME = 'test_retrieve_bigquery_param_sql'
dag = DAG(DAG_NAME, default_args=default_args)

test_task_retrieve_sql = BigQueryOperator(
    task_id='test_task_retrieve_sql',
    sql=['select * from test_table'],
    dag=dag,
    default_args=default_args,
    schema_update_options=None
)



from kueri.models.dag_bq_query_extractor import DagBqQueryExtractor
from .base_command import BaseCommand
import logging


class DagBqExtractorCommand(BaseCommand):

    def execute(self):
        """
            Main to execute DAGBqQueryExtractor
        """

        dag_bq_extractor = DagBqQueryExtractor(**self.payload)
        try:
            dag_object = dag_bq_extractor.read_dag_scripts()
            bq_operator_dict = dag_bq_extractor.filter_bigquery_operator_task(dag_object)
            query = dag_bq_extractor.retrieve_bigquery_operator_attribute(bq_operator_dict)
            print(query)
        except Exception as e:
            raise e
        return query



from airflow.models import DAG
from datetime import datetime

DEFAULT_DATE = datetime(2015, 1, 1)
default_args = {'owner': 'de-team', 'start_date': DEFAULT_DATE}
DAG_NAME = 'test_no_bigquery_operators_found'
dag = DAG(DAG_NAME, default_args=default_args)



import importlib
import sys
import logging


class CommandFactory:

    CLASS_SUFFIX = 'Command'
    FILE_SUFFIX = '_command'
    BASE_MODULE = 'kueri.commands'

    def __init__(self, command_string):
        self.command_string = command_string
        self.command_class = self._import_class()

    def _class_name(self):
        """
            Parsing name module in to camelcase then join with const FILE_SUFFIX
            :return: Class name of module
        """

        command_string_camelcase = ''.join(x.title() for x in self.command_string.split('_'))
        return command_string_camelcase + self.CLASS_SUFFIX

    def _import_class(self):
        """
            Import class and assign attribute of the class based on input
            :return: Object of class
        """

        try:
            mod = importlib.import_module('{}.{}{}'.format(self.BASE_MODULE, self.command_string, self.FILE_SUFFIX))
            cls = getattr(mod, self._class_name())
        except ModuleNotFoundError:
            logging.info('Invalid Command')
            sys.exit(1)
        except AttributeError:
            raise AttributeError
            logging.info('Incorrect command class name. Contact DE Team')
            sys.exit(1)
        return cls

    def create(self, payload):
        """
            Crate object of class
            :param payload: Input parameter as attribute of class
            :return:
        """

        return self.command_class(payload)



from google.cloud import bigquery
from google.api_core.exceptions import GoogleAPICallError


class QueryValidator:

    def __init__(self, query, project_id):
        self.query = query
        self.project_id = project_id

    def validate_query(self):
        """
            Validate query
            :return: String valid query or not
        """
        client = bigquery.Client(project=self.project_id)
        job_config = bigquery.QueryJobConfig()
        job_config.use_query_cache = False
        job_config.use_legacy_sql = False
        job_config.dry_run = True
        try:
            client.query(
                query=self.query,
                location="US",
                job_config=job_config,
            )
        except GoogleAPICallError as google_exceptions:
            return "invalid query with value = {}".format(google_exceptions.errors[0].get('message'))
        return "valid query"



import abc


class BaseCommand(abc.ABC):

    def __init__(self, payload):
        self.payload = payload

    @abc.abstractmethod
    def execute(self):
        """
            Abstract Method to execute models
        """
        pass



import logging
import importlib
import sys
import os
import sqlparse
import json
import re


class DagBqQueryExtractor:
    
    def __init__(self, file_name, path_to_folder_file):
        self.file_name = file_name
        self.path_to_folder_file = path_to_folder_file
        self.bq_transformation_query = [
            {"name": "transformation_query", "type": "STRING"},
            {"name": "destination_table", "type": "STRING"}
        ]

    def read_dag_scripts(self):
        """
            Import module from python file
            :return: module python
        """

        dag_object = {}
        sys.path.append(os.path.abspath(self.path_to_folder_file))
        module_name = self.file_name[:-3]
        try:
            dag_object[module_name] = importlib.import_module(module_name)
        except ModuleNotFoundError:
            raise ModuleNotFoundError("Module Not Found")
        return dag_object

    @staticmethod
    def filter_bigquery_operator_task(dag_object):
        """
            Create dictionary task and BigQueryOperator
            :param dag_object: Python Module
            :return: Dictionary
        """

        logging.info('Filtering valid BigQueryOperator tasks')
        bq_operator_dict = {}
        for module_name, module_content in dag_object.items():
            module_objects = module_content.__dict__
            for key, value in module_objects.items():
                if str(type(value)) == "<class 'airflow.contrib.operators.bigquery_operator.BigQueryOperator'>":
                    bq_operator_dict[key] = value
        return bq_operator_dict

    @staticmethod
    def retrieve_bigquery_operator_attribute(bq_operator_dict):
        """
            Retrieve sql from Dictionary bq_operator_dict
            :param bq_operator_dict: Dictionary of task name and it's BigQueryOperator
            :return: string
        """

        dict_task_query = {}
        logging.info('Retrieve BigQueryOperator attributes')
        for key, value in bq_operator_dict.items():
            if value.bql:
                query = value.bql
                query = query.replace('\n', '')
                query = re.sub(' +', ' ', query)
            else:
                query = value.sql
                query = '.'.join(query)
                query = re.sub(' +', ' ', query)
            dict_task_query[key] = query
        return json.dumps(dict_task_query)



from kueri.models.query_validator import QueryValidator
from .base_command import BaseCommand
import logging


class QueryValidatorCommand(BaseCommand):

    def execute(self):
        """
            Main to execute QueryValidator
        """

        query_validator = QueryValidator(**self.payload)
        result = query_validator.validate_query()
        print("result = {}".format(result))
        return result



#!/usr/bin/python
import click
import coloredlogs
from kueri.commands.command_factory import CommandFactory


def parse_args(arguments):
    """
        Create dict of argument and value
        :return dictionary of payload
    """

    payload = {}
    for argument in arguments:
        argument = argument.split('=')
        key = argument[0]
        value = "=".join(argument[1:])
        payload[key] = value
    return payload


@click.command()
@click.argument('arguments', nargs=-1)
def main(arguments):
    """
        Execute the command
    """
    coloredlogs.install(fmt='%(levelname)s:%(message)s', level='INFO')
    payload = parse_args(arguments)
    command_string = payload.pop('cmd')
    command = CommandFactory(command_string).create(payload)
    command.execute()
